# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .billing_provider_allocation_target import BillingProviderAllocationTarget
from .claim_allocation_target import ClaimAllocationTarget
from .service_line_allocation_target import ServiceLineAllocationTarget

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AllocationTarget_ServiceLine(ServiceLineAllocationTarget):
    type: typing_extensions.Literal["service_line"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class AllocationTarget_Claim(ClaimAllocationTarget):
    type: typing_extensions.Literal["claim"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class AllocationTarget_BillingProviderId(BillingProviderAllocationTarget):
    type: typing_extensions.Literal["billing_provider_id"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class AllocationTarget_Unattributed(pydantic.BaseModel):
    type: typing_extensions.Literal["unattributed"]

    class Config:
        frozen = True
        smart_union = True


AllocationTarget = typing.Union[
    AllocationTarget_ServiceLine,
    AllocationTarget_Claim,
    AllocationTarget_BillingProviderId,
    AllocationTarget_Unattributed,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .authorized_signatory import AuthorizedSignatory

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AuthorizedSignatoryUpdate_Set(AuthorizedSignatory):
    type: typing_extensions.Literal["set"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class AuthorizedSignatoryUpdate_Remove(pydantic.BaseModel):
    type: typing_extensions.Literal["remove"]

    class Config:
        frozen = True
        smart_union = True


AuthorizedSignatoryUpdate = typing.Union[AuthorizedSignatoryUpdate_Set, AuthorizedSignatoryUpdate_Remove]



# This file was auto-generated by Fern from our API Definition.

from .authorized_signatory import AuthorizedSignatory
from .authorized_signatory_update import (
    AuthorizedSignatoryUpdate,
    AuthorizedSignatoryUpdate_Remove,
    AuthorizedSignatoryUpdate_Set,
)
from .contract import Contract
from .contract_base import ContractBase
from .contract_id import ContractId
from .contract_is_linked_to_fee_schedule_error import ContractIsLinkedToFeeScheduleError
from .contract_status import ContractStatus
from .contract_with_providers import ContractWithProviders
from .contracting_provider_id import ContractingProviderId
from .contracts_page import ContractsPage
from .date_update import DateUpdate, DateUpdate_Remove, DateUpdate_Set
from .insurance_types import (
    InsuranceTypes,
    InsuranceTypes_AllApply,
    InsuranceTypes_NoneApply,
    InsuranceTypes_TheseApply,
)
from .regions_update import RegionsUpdate, RegionsUpdate_Remove, RegionsUpdate_Set
from .rendering_providerid import RenderingProviderid

__all__ = [
    "AuthorizedSignatory",
    "AuthorizedSignatoryUpdate",
    "AuthorizedSignatoryUpdate_Remove",
    "AuthorizedSignatoryUpdate_Set",
    "Contract",
    "ContractBase",
    "ContractId",
    "ContractIsLinkedToFeeScheduleError",
    "ContractStatus",
    "ContractWithProviders",
    "ContractingProviderId",
    "ContractsPage",
    "DateUpdate",
    "DateUpdate_Remove",
    "DateUpdate_Set",
    "InsuranceTypes",
    "InsuranceTypes_AllApply",
    "InsuranceTypes_NoneApply",
    "InsuranceTypes_TheseApply",
    "RegionsUpdate",
    "RegionsUpdate_Remove",
    "RegionsUpdate_Set",
    "RenderingProviderid",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.street_address_long_zip import StreetAddressLongZip
from .encounter_provider_base import EncounterProviderBase
from .provider_id import ProviderId


class EncounterProvider(EncounterProviderBase):
    """
    import uuid

    from candid import State, StreetAddressLongZip
    from candid.resources.encounter_providers.v_2 import EncounterProvider

    EncounterProvider(
        provider_id=uuid.UUID(
            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
        ),
        address=StreetAddressLongZip(
            address_1="123 Main St",
            address_2="Apt 1",
            city="New York",
            state=State.NY,
            zip_code="10001",
            zip_plus_four_code="1234",
        ),
        tax_id="123456789",
        npi="1234567890",
        taxonomy_code="207Q00000X",
        first_name="John",
        last_name="Doe",
        organization_name="Organization Name",
    )
    """

    provider_id: ProviderId
    address: StreetAddressLongZip
    tax_id: typing.Optional[str] = None
    npi: str
    taxonomy_code: typing.Optional[str] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.service_line_id import ServiceLineId
from .patient_write_off_reason import PatientWriteOffReason

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class PatientWriteOffCreate(pydantic.BaseModel):
    write_off_timestamp: dt.datetime
    write_off_note: typing.Optional[str] = None
    write_off_reason: PatientWriteOffReason
    service_line_id: ServiceLineId
    amount_cents: int

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.claim_id import ClaimId
from .....commons.types.date_range_optional_end import DateRangeOptionalEnd
from .....commons.types.decimal import Decimal
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.procedure_modifier import ProcedureModifier
from .....commons.types.service_line_id import ServiceLineId
from .....commons.types.service_line_units import ServiceLineUnits
from .....diagnoses.types.diagnosis_id import DiagnosisId
from .....invoices.types.invoice import Invoice
from .service_line_adjustment import ServiceLineAdjustment
from .service_line_denial_reason import ServiceLineDenialReason
from .service_line_era_data import ServiceLineEraData

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ServiceLine(pydantic.BaseModel):
    """
    import datetime
    import uuid

    from candid import (DateRangeOptionalEnd, FacilityTypeCode, Invoice,
                        InvoiceItem, InvoiceStatus, ProcedureModifier,
                        ServiceLineUnits)
    from candid.resources.service_lines.v_2 import (DenialReasonContent,
                                                    ServiceLine,
                                                    ServiceLineAdjustment,
                                                    ServiceLineDenialReason,
                                                    ServiceLineEraData)

    ServiceLine(modifiers=[ProcedureModifier.22], charge_amount_cents=10000, allowed_amount_cents=8000, insurance_balance_cents=0, patient_balance_cents=2000, paid_amount_cents=8000, patient_responsibility_cents=2000, diagnosis_id_zero=uuid.UUID("4ac84bcd-12f5-4f86-a57b-e06749127c98", ), diagnosis_id_one=uuid.UUID("eea5ca5a-8b43-45fd-8cbd-c6cc1103e759", ), diagnosis_id_two=uuid.UUID("5c4aa029-2db9-4202-916e-e93c708f65ff", ), diagnosis_id_three=uuid.UUID("81795126-a3ac-443c-b47e-7259a16ab4a2", ), service_line_era_data=ServiceLineEraData(service_line_adjustments=[ServiceLineAdjustment(created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), adjustment_group_code="CO", adjustment_reason_code="CO", adjustment_amount_cents=1000, adjustment_note="test_note", )], remittance_advice_remark_codes=["N362"], ), service_line_manual_adjustments=[ServiceLineAdjustment(created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), adjustment_group_code="CO", adjustment_reason_code="CO", adjustment_amount_cents=1000, adjustment_note="test_note", )], related_invoices=[Invoice(id=uuid.UUID("901be2f1-41bc-456e-9987-4fe2f84f9d75", ), created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), updated_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), organzation_id=uuid.UUID("f13f73d4-4344-46ea-9d93-33bcffbb9f36", ), source_id="9B626577-8808-4F28-9ED1-F0DFF0D49BBC", source_customer_id="624D1972-8C69-4C2F-AEFA-10856F734DB3", patient_external_id="10FED4D6-4C5A-48DF-838A-EEF45A74788D", note="test_note", due_date="2023-10-10", status=InvoiceStatus.DRAFT, url="https://example.com", customer_invoice_url="https://example.com", items=[InvoiceItem(service_line_id=uuid.UUID("ced00f23-6e68-4678-9dbc-f5aa2969a565", ), amount_cents=500, )], )], denial_reason=ServiceLineDenialReason(reason=DenialReasonContent.AUTHORIZATION_REQUIRED, ), place_of_service_code=FacilityTypeCode.01, service_line_id=uuid.UUID("ced00f23-6e68-4678-9dbc-f5aa2969a565", ), procedure_code="99213", quantity="1", units=ServiceLineUnits.MJ, claim_id=uuid.UUID("026a1fb8-748e-4859-a2d7-3ea9e07d25ae", ), date_of_service_range=DateRangeOptionalEnd(start_date="2023-01-01", end_date="2023-01-03", ), )
    """

    modifiers: typing.Optional[typing.List[ProcedureModifier]] = None
    charge_amount_cents: typing.Optional[int] = None
    allowed_amount_cents: typing.Optional[int] = None
    insurance_balance_cents: typing.Optional[int] = None
    patient_balance_cents: typing.Optional[int] = None
    paid_amount_cents: typing.Optional[int] = None
    patient_responsibility_cents: typing.Optional[int] = None
    diagnosis_id_zero: typing.Optional[DiagnosisId] = None
    diagnosis_id_one: typing.Optional[DiagnosisId] = None
    diagnosis_id_two: typing.Optional[DiagnosisId] = None
    diagnosis_id_three: typing.Optional[DiagnosisId] = None
    service_line_era_data: typing.Optional[ServiceLineEraData] = None
    service_line_manual_adjustments: typing.Optional[typing.List[ServiceLineAdjustment]] = None
    related_invoices: typing.Optional[typing.List[Invoice]] = None
    denial_reason: typing.Optional[ServiceLineDenialReason] = None
    place_of_service_code: typing.Optional[FacilityTypeCode] = None
    service_line_id: ServiceLineId
    procedure_code: str
    quantity: Decimal = pydantic.Field(
        description=(
            "String representation of a Decimal that can be parsed by most libraries.\n"
            "A ServiceLine quantity cannot contain more than one digit of precision.\n"
            "Example: 1.1 is valid, 1.11 is not.\n"
        )
    )
    units: ServiceLineUnits
    claim_id: ClaimId
    date_of_service_range: DateRangeOptionalEnd = pydantic.Field(
        description=(
            "A range of dates of service for this service line. If the service line is for a single date, the end date\n"
            "will be empty.\n"
        )
    )
    description: typing.Optional[str] = pydantic.Field(
        default=None,
        description="A free-form description to clarify the related data elements and their content. Maps to SV1-01, C003-07 on the 837-P.",
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .compute_all_in_network_providers_request import ComputeAllInNetworkProvidersRequest
from .compute_all_in_network_providers_response import ComputeAllInNetworkProvidersResponse
from .compute_all_in_network_rendering_providers_result import (
    ComputeAllInNetworkRenderingProvidersResult,
    ComputeAllInNetworkRenderingProvidersResult_Indeterminate,
    ComputeAllInNetworkRenderingProvidersResult_OutOfNetwork,
    ComputeAllInNetworkRenderingProvidersResult_RenderingProviders,
)
from .expected_network_status_check_error_message import ExpectedNetworkStatusCheckErrorMessage
from .expected_network_status_request_v_2 import ExpectedNetworkStatusRequestV2
from .expected_network_status_response_v_2 import ExpectedNetworkStatusResponseV2
from .expected_network_status_subscriber_information import ExpectedNetworkStatusSubscriberInformation
from .expected_network_status_v_2 import (
    ExpectedNetworkStatusV2,
    ExpectedNetworkStatusV2_InNetwork,
    ExpectedNetworkStatusV2_Indeterminate,
    ExpectedNetworkStatusV2_OutOfNetwork,
)
from .explanation import Explanation
from .in_network_rendering_providers_detail import InNetworkRenderingProvidersDetail
from .in_network_status import InNetworkStatus
from .indeterminate_network_status import IndeterminateNetworkStatus
from .insurance_type import InsuranceType
from .insurance_type_codes import (
    InsuranceTypeCodes,
    InsuranceTypeCodes_InsuranceTypeCode,
    InsuranceTypeCodes_NotApplicable,
    InsuranceTypeCodes_UnknownInsuranceTypeCode,
)
from .line_of_business import LineOfBusiness
from .network_status_check_id import NetworkStatusCheckId
from .out_of_network_status import OutOfNetworkStatus
from .service_type import ServiceType

__all__ = [
    "ComputeAllInNetworkProvidersRequest",
    "ComputeAllInNetworkProvidersResponse",
    "ComputeAllInNetworkRenderingProvidersResult",
    "ComputeAllInNetworkRenderingProvidersResult_Indeterminate",
    "ComputeAllInNetworkRenderingProvidersResult_OutOfNetwork",
    "ComputeAllInNetworkRenderingProvidersResult_RenderingProviders",
    "ExpectedNetworkStatusCheckErrorMessage",
    "ExpectedNetworkStatusRequestV2",
    "ExpectedNetworkStatusResponseV2",
    "ExpectedNetworkStatusSubscriberInformation",
    "ExpectedNetworkStatusV2",
    "ExpectedNetworkStatusV2_InNetwork",
    "ExpectedNetworkStatusV2_Indeterminate",
    "ExpectedNetworkStatusV2_OutOfNetwork",
    "Explanation",
    "InNetworkRenderingProvidersDetail",
    "InNetworkStatus",
    "IndeterminateNetworkStatus",
    "InsuranceType",
    "InsuranceTypeCodes",
    "InsuranceTypeCodes_InsuranceTypeCode",
    "InsuranceTypeCodes_NotApplicable",
    "InsuranceTypeCodes_UnknownInsuranceTypeCode",
    "LineOfBusiness",
    "NetworkStatusCheckId",
    "OutOfNetworkStatus",
    "ServiceType",
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
import uuid
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_dict import remove_none_from_dict
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.date import Date
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.regions import Regions
from ....commons.types.state import State
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from .errors.contract_is_linked_to_fee_schedule_http_error import ContractIsLinkedToFeeScheduleHttpError
from .types.authorized_signatory import AuthorizedSignatory
from .types.authorized_signatory_update import AuthorizedSignatoryUpdate
from .types.contract_id import ContractId
from .types.contract_is_linked_to_fee_schedule_error import ContractIsLinkedToFeeScheduleError
from .types.contract_status import ContractStatus
from .types.contract_with_providers import ContractWithProviders
from .types.contracting_provider_id import ContractingProviderId
from .types.contracts_page import ContractsPage
from .types.date_update import DateUpdate
from .types.insurance_types import InsuranceTypes
from .types.regions_update import RegionsUpdate
from .types.rendering_providerid import RenderingProviderid

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, contract_id: ContractId) -> ContractWithProviders:
        """
        Parameters:
            - contract_id: ContractId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/contracts/v2/{contract_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ContractWithProviders, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        contracting_provider_id: typing.Optional[ContractingProviderId] = None,
        rendering_provider_ids: typing.Optional[
            typing.Union[RenderingProviderid, typing.List[RenderingProviderid]]
        ] = None,
        payer_names: typing.Optional[typing.Union[str, typing.List[str]]] = None,
        states: typing.Optional[typing.Union[State, typing.List[State]]] = None,
        contract_status: typing.Optional[ContractStatus] = None,
    ) -> ContractsPage:
        """
        Parameters:
            - page_token: typing.Optional[PageToken].

            - limit: typing.Optional[int]. Max number of contracts returned. Defaults to 1000. Max is 1000.

            - contracting_provider_id: typing.Optional[ContractingProviderId].

            - rendering_provider_ids: typing.Optional[typing.Union[RenderingProviderid, typing.List[RenderingProviderid]]].

            - payer_names: typing.Optional[typing.Union[str, typing.List[str]]]. Filter to contracts that include any of the included payer names.

            - states: typing.Optional[typing.Union[State, typing.List[State]]].

            - contract_status: typing.Optional[ContractStatus]. The status of the contract. Defaults to `pending`
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/contracts/v2"),
            params=remove_none_from_dict(
                {
                    "page_token": page_token,
                    "limit": limit,
                    "contracting_provider_id": jsonable_encoder(contracting_provider_id),
                    "rendering_provider_ids": jsonable_encoder(rendering_provider_ids),
                    "payer_names": payer_names,
                    "states": states,
                    "contract_status": contract_status,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ContractsPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        contracting_provider_id: ContractingProviderId,
        rendering_provider_ids: typing.List[RenderingProviderid],
        payer_uuid: uuid.UUID,
        effective_date: Date,
        expiration_date: typing.Optional[Date] = OMIT,
        regions: Regions,
        contract_status: typing.Optional[ContractStatus] = OMIT,
        authorized_signatory: typing.Optional[AuthorizedSignatory] = OMIT,
        commercial_insurance_types: InsuranceTypes,
        medicare_insurance_types: InsuranceTypes,
        medicaid_insurance_types: InsuranceTypes,
    ) -> ContractWithProviders:
        """
        Creates a new contract within the user's current organization

        Parameters:
            - contracting_provider_id: ContractingProviderId. The UUID of the provider under agreement to the contract

            - rendering_provider_ids: typing.List[RenderingProviderid]. A rendering provider isn't contracted directly with the payer but can render
                                                                        services under the contract held by the contracting provider.
                                                                        Max items is 100.

            - payer_uuid: uuid.UUID. The UUID of the insurance company under agreement to the contract

            - effective_date: Date. The starting day upon which the contract is effective

            - expiration_date: typing.Optional[Date]. An optional end day upon which the contract expires

            - regions: Regions. The state(s) to which the contract's coverage extends.
                                It may also be set to "national" for the entirety of the US.
            - contract_status: typing.Optional[ContractStatus].

            - authorized_signatory: typing.Optional[AuthorizedSignatory].

            - commercial_insurance_types: InsuranceTypes. The commercial plan insurance types this contract applies.

            - medicare_insurance_types: InsuranceTypes. The Medicare plan insurance types this contract applies.

            - medicaid_insurance_types: InsuranceTypes. The Medicaid plan insurance types this contract applies.
        """
        _request: typing.Dict[str, typing.Any] = {
            "contracting_provider_id": contracting_provider_id,
            "rendering_provider_ids": rendering_provider_ids,
            "payer_uuid": payer_uuid,
            "effective_date": effective_date,
            "regions": regions,
            "commercial_insurance_types": commercial_insurance_types,
            "medicare_insurance_types": medicare_insurance_types,
            "medicaid_insurance_types": medicaid_insurance_types,
        }
        if expiration_date is not OMIT:
            _request["expiration_date"] = expiration_date
        if contract_status is not OMIT:
            _request["contract_status"] = contract_status
        if authorized_signatory is not OMIT:
            _request["authorized_signatory"] = authorized_signatory
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/contracts/v2"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ContractWithProviders, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, contract_id: ContractId) -> None:
        """
        Parameters:
            - contract_id: ContractId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/contracts/v2/{contract_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "ContractIsLinkedToFeeScheduleHttpError":
                raise ContractIsLinkedToFeeScheduleHttpError(
                    pydantic.parse_obj_as(ContractIsLinkedToFeeScheduleError, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        contract_id: ContractId,
        *,
        rendering_provider_ids: typing.Optional[typing.List[RenderingProviderid]] = OMIT,
        effective_date: typing.Optional[Date] = OMIT,
        expiration_date: typing.Optional[DateUpdate] = OMIT,
        regions: typing.Optional[RegionsUpdate] = OMIT,
        contract_status: typing.Optional[ContractStatus] = OMIT,
        authorized_signatory: typing.Optional[AuthorizedSignatoryUpdate] = OMIT,
        commercial_insurance_types: typing.Optional[InsuranceTypes] = OMIT,
        medicare_insurance_types: typing.Optional[InsuranceTypes] = OMIT,
        medicaid_insurance_types: typing.Optional[InsuranceTypes] = OMIT,
    ) -> ContractWithProviders:
        """
        Parameters:
            - contract_id: ContractId.

            - rendering_provider_ids: typing.Optional[typing.List[RenderingProviderid]]. A rendering provider isn't contracted directly with the payer but can render
                                                                                         services under the contract held by the contracting provider.
                                                                                         Max items is 100.

            - effective_date: typing.Optional[Date]. The starting day upon which the contract is effective

            - expiration_date: typing.Optional[DateUpdate]. An optional end day upon which the contract expires

            - regions: typing.Optional[RegionsUpdate]. If present, the contract's rendering providers will be patched to this exact
                                                       value, overriding what was set before.

            - contract_status: typing.Optional[ContractStatus].

            - authorized_signatory: typing.Optional[AuthorizedSignatoryUpdate].

            - commercial_insurance_types: typing.Optional[InsuranceTypes].

            - medicare_insurance_types: typing.Optional[InsuranceTypes].

            - medicaid_insurance_types: typing.Optional[InsuranceTypes].
        """
        _request: typing.Dict[str, typing.Any] = {}
        if rendering_provider_ids is not OMIT:
            _request["rendering_provider_ids"] = rendering_provider_ids
        if effective_date is not OMIT:
            _request["effective_date"] = effective_date
        if expiration_date is not OMIT:
            _request["expiration_date"] = expiration_date
        if regions is not OMIT:
            _request["regions"] = regions
        if contract_status is not OMIT:
            _request["contract_status"] = contract_status
        if authorized_signatory is not OMIT:
            _request["authorized_signatory"] = authorized_signatory
        if commercial_insurance_types is not OMIT:
            _request["commercial_insurance_types"] = commercial_insurance_types
        if medicare_insurance_types is not OMIT:
            _request["medicare_insurance_types"] = medicare_insurance_types
        if medicaid_insurance_types is not OMIT:
            _request["medicaid_insurance_types"] = medicaid_insurance_types
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/contracts/v2/{contract_id}"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ContractWithProviders, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(self, contract_id: ContractId) -> ContractWithProviders:
        """
        Parameters:
            - contract_id: ContractId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/contracts/v2/{contract_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ContractWithProviders, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        contracting_provider_id: typing.Optional[ContractingProviderId] = None,
        rendering_provider_ids: typing.Optional[
            typing.Union[RenderingProviderid, typing.List[RenderingProviderid]]
        ] = None,
        payer_names: typing.Optional[typing.Union[str, typing.List[str]]] = None,
        states: typing.Optional[typing.Union[State, typing.List[State]]] = None,
        contract_status: typing.Optional[ContractStatus] = None,
    ) -> ContractsPage:
        """
        Parameters:
            - page_token: typing.Optional[PageToken].

            - limit: typing.Optional[int]. Max number of contracts returned. Defaults to 1000. Max is 1000.

            - contracting_provider_id: typing.Optional[ContractingProviderId].

            - rendering_provider_ids: typing.Optional[typing.Union[RenderingProviderid, typing.List[RenderingProviderid]]].

            - payer_names: typing.Optional[typing.Union[str, typing.List[str]]]. Filter to contracts that include any of the included payer names.

            - states: typing.Optional[typing.Union[State, typing.List[State]]].

            - contract_status: typing.Optional[ContractStatus]. The status of the contract. Defaults to `pending`
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/contracts/v2"),
            params=remove_none_from_dict(
                {
                    "page_token": page_token,
                    "limit": limit,
                    "contracting_provider_id": jsonable_encoder(contracting_provider_id),
                    "rendering_provider_ids": jsonable_encoder(rendering_provider_ids),
                    "payer_names": payer_names,
                    "states": states,
                    "contract_status": contract_status,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ContractsPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        contracting_provider_id: ContractingProviderId,
        rendering_provider_ids: typing.List[RenderingProviderid],
        payer_uuid: uuid.UUID,
        effective_date: Date,
        expiration_date: typing.Optional[Date] = OMIT,
        regions: Regions,
        contract_status: typing.Optional[ContractStatus] = OMIT,
        authorized_signatory: typing.Optional[AuthorizedSignatory] = OMIT,
        commercial_insurance_types: InsuranceTypes,
        medicare_insurance_types: InsuranceTypes,
        medicaid_insurance_types: InsuranceTypes,
    ) -> ContractWithProviders:
        """
        Creates a new contract within the user's current organization

        Parameters:
            - contracting_provider_id: ContractingProviderId. The UUID of the provider under agreement to the contract

            - rendering_provider_ids: typing.List[RenderingProviderid]. A rendering provider isn't contracted directly with the payer but can render
                                                                        services under the contract held by the contracting provider.
                                                                        Max items is 100.

            - payer_uuid: uuid.UUID. The UUID of the insurance company under agreement to the contract

            - effective_date: Date. The starting day upon which the contract is effective

            - expiration_date: typing.Optional[Date]. An optional end day upon which the contract expires

            - regions: Regions. The state(s) to which the contract's coverage extends.
                                It may also be set to "national" for the entirety of the US.
            - contract_status: typing.Optional[ContractStatus].

            - authorized_signatory: typing.Optional[AuthorizedSignatory].

            - commercial_insurance_types: InsuranceTypes. The commercial plan insurance types this contract applies.

            - medicare_insurance_types: InsuranceTypes. The Medicare plan insurance types this contract applies.

            - medicaid_insurance_types: InsuranceTypes. The Medicaid plan insurance types this contract applies.
        """
        _request: typing.Dict[str, typing.Any] = {
            "contracting_provider_id": contracting_provider_id,
            "rendering_provider_ids": rendering_provider_ids,
            "payer_uuid": payer_uuid,
            "effective_date": effective_date,
            "regions": regions,
            "commercial_insurance_types": commercial_insurance_types,
            "medicare_insurance_types": medicare_insurance_types,
            "medicaid_insurance_types": medicaid_insurance_types,
        }
        if expiration_date is not OMIT:
            _request["expiration_date"] = expiration_date
        if contract_status is not OMIT:
            _request["contract_status"] = contract_status
        if authorized_signatory is not OMIT:
            _request["authorized_signatory"] = authorized_signatory
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/contracts/v2"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ContractWithProviders, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, contract_id: ContractId) -> None:
        """
        Parameters:
            - contract_id: ContractId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/contracts/v2/{contract_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "ContractIsLinkedToFeeScheduleHttpError":
                raise ContractIsLinkedToFeeScheduleHttpError(
                    pydantic.parse_obj_as(ContractIsLinkedToFeeScheduleError, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        contract_id: ContractId,
        *,
        rendering_provider_ids: typing.Optional[typing.List[RenderingProviderid]] = OMIT,
        effective_date: typing.Optional[Date] = OMIT,
        expiration_date: typing.Optional[DateUpdate] = OMIT,
        regions: typing.Optional[RegionsUpdate] = OMIT,
        contract_status: typing.Optional[ContractStatus] = OMIT,
        authorized_signatory: typing.Optional[AuthorizedSignatoryUpdate] = OMIT,
        commercial_insurance_types: typing.Optional[InsuranceTypes] = OMIT,
        medicare_insurance_types: typing.Optional[InsuranceTypes] = OMIT,
        medicaid_insurance_types: typing.Optional[InsuranceTypes] = OMIT,
    ) -> ContractWithProviders:
        """
        Parameters:
            - contract_id: ContractId.

            - rendering_provider_ids: typing.Optional[typing.List[RenderingProviderid]]. A rendering provider isn't contracted directly with the payer but can render
                                                                                         services under the contract held by the contracting provider.
                                                                                         Max items is 100.

            - effective_date: typing.Optional[Date]. The starting day upon which the contract is effective

            - expiration_date: typing.Optional[DateUpdate]. An optional end day upon which the contract expires

            - regions: typing.Optional[RegionsUpdate]. If present, the contract's rendering providers will be patched to this exact
                                                       value, overriding what was set before.

            - contract_status: typing.Optional[ContractStatus].

            - authorized_signatory: typing.Optional[AuthorizedSignatoryUpdate].

            - commercial_insurance_types: typing.Optional[InsuranceTypes].

            - medicare_insurance_types: typing.Optional[InsuranceTypes].

            - medicaid_insurance_types: typing.Optional[InsuranceTypes].
        """
        _request: typing.Dict[str, typing.Any] = {}
        if rendering_provider_ids is not OMIT:
            _request["rendering_provider_ids"] = rendering_provider_ids
        if effective_date is not OMIT:
            _request["effective_date"] = effective_date
        if expiration_date is not OMIT:
            _request["expiration_date"] = expiration_date
        if regions is not OMIT:
            _request["regions"] = regions
        if contract_status is not OMIT:
            _request["contract_status"] = contract_status
        if authorized_signatory is not OMIT:
            _request["authorized_signatory"] = authorized_signatory
        if commercial_insurance_types is not OMIT:
            _request["commercial_insurance_types"] = commercial_insurance_types
        if medicare_insurance_types is not OMIT:
            _request["medicare_insurance_types"] = medicare_insurance_types
        if medicaid_insurance_types is not OMIT:
            _request["medicaid_insurance_types"] = medicaid_insurance_types
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/contracts/v2/{contract_id}"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ContractWithProviders, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClaimFrequencyTypeCode(str, enum.Enum):
    ORIGINAL = "1"
    CORRECTED = "6"
    REPLACEMENT = "7"
    VOID = "8"

    def visit(
        self,
        original: typing.Callable[[], T_Result],
        corrected: typing.Callable[[], T_Result],
        replacement: typing.Callable[[], T_Result],
        void: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClaimFrequencyTypeCode.ORIGINAL:
            return original()
        if self is ClaimFrequencyTypeCode.CORRECTED:
            return corrected()
        if self is ClaimFrequencyTypeCode.REPLACEMENT:
            return replacement()
        if self is ClaimFrequencyTypeCode.VOID:
            return void()



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from ....era.errors.era_not_fully_processed_error import EraNotFullyProcessedError
from ....era.types.era_not_fully_processed_error_message import EraNotFullyProcessedErrorMessage
from .types.insurance_adjudication import InsuranceAdjudication
from .types.insurance_adjudication_create import InsuranceAdjudicationCreate
from .types.insurance_adjudication_id import InsuranceAdjudicationId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, insurance_adjudication_id: InsuranceAdjudicationId) -> InsuranceAdjudication:
        """
        Retrieves a previously created insurance adjudication by its `insurance_adjudication_id`.

        Parameters:
            - insurance_adjudication_id: InsuranceAdjudicationId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/insurance-adjudications/v1/{insurance_adjudication_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceAdjudication, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, *, request: InsuranceAdjudicationCreate) -> InsuranceAdjudication:
        """
        Creates a new insurance adjudication record and returns the newly created InsuranceAdjudication object.

        Parameters:
            - request: InsuranceAdjudicationCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/insurance-adjudications/v1"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceAdjudication, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EraNotFullyProcessedError":
                raise EraNotFullyProcessedError(
                    pydantic.parse_obj_as(EraNotFullyProcessedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, insurance_adjudication_id: InsuranceAdjudicationId) -> None:
        """
        Deletes the insurance adjudication record matching the provided insurance_adjudication_id.

        Parameters:
            - insurance_adjudication_id: InsuranceAdjudicationId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/insurance-adjudications/v1/{insurance_adjudication_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(self, insurance_adjudication_id: InsuranceAdjudicationId) -> InsuranceAdjudication:
        """
        Retrieves a previously created insurance adjudication by its `insurance_adjudication_id`.

        Parameters:
            - insurance_adjudication_id: InsuranceAdjudicationId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/insurance-adjudications/v1/{insurance_adjudication_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceAdjudication, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(self, *, request: InsuranceAdjudicationCreate) -> InsuranceAdjudication:
        """
        Creates a new insurance adjudication record and returns the newly created InsuranceAdjudication object.

        Parameters:
            - request: InsuranceAdjudicationCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/insurance-adjudications/v1"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceAdjudication, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EraNotFullyProcessedError":
                raise EraNotFullyProcessedError(
                    pydantic.parse_obj_as(EraNotFullyProcessedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, insurance_adjudication_id: InsuranceAdjudicationId) -> None:
        """
        Deletes the insurance adjudication record matching the provided insurance_adjudication_id.

        Parameters:
            - insurance_adjudication_id: InsuranceAdjudicationId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/insurance-adjudications/v1/{insurance_adjudication_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_dict import remove_none_from_dict
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.claim_id import ClaimId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.invoice_id import InvoiceId
from ....commons.types.page_token import PageToken
from ....commons.types.patient_external_id import PatientExternalId
from ....commons.types.provider_id import ProviderId
from ....commons.types.service_line_id import ServiceLineId
from ....commons.types.sort_direction import SortDirection
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from ....financials.types.allocation_create import AllocationCreate
from ....financials.types.invoice_update import InvoiceUpdate
from ....financials.types.note_update import NoteUpdate
from ....financials.types.patient_transaction_source import PatientTransactionSource
from .types.patient_payment import PatientPayment
from .types.patient_payment_id import PatientPaymentId
from .types.patient_payment_sort_field import PatientPaymentSortField
from .types.patient_payments_page import PatientPaymentsPage

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V4Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        patient_external_id: typing.Optional[PatientExternalId] = None,
        claim_id: typing.Optional[ClaimId] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        unattributed: typing.Optional[bool] = None,
        invoice_id: typing.Optional[InvoiceId] = None,
        sources: typing.Optional[typing.Union[PatientTransactionSource, typing.List[PatientTransactionSource]]] = None,
        sort: typing.Optional[PatientPaymentSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
    ) -> PatientPaymentsPage:
        """
        Returns all patient payments satisfying the search criteria AND whose organization_id matches
        the current organization_id of the authenticated user.

        Parameters:
            - limit: typing.Optional[int]. Defaults to 100. The value must be greater than 0 and less than 1000.

            - patient_external_id: typing.Optional[PatientExternalId].

            - claim_id: typing.Optional[ClaimId].

            - service_line_id: typing.Optional[ServiceLineId].

            - billing_provider_id: typing.Optional[ProviderId].

            - unattributed: typing.Optional[bool]. returns payments with unattributed allocations if set to true

            - invoice_id: typing.Optional[InvoiceId].

            - sources: typing.Optional[typing.Union[PatientTransactionSource, typing.List[PatientTransactionSource]]].

            - sort: typing.Optional[PatientPaymentSortField]. Defaults to payment_timestamp

            - sort_direction: typing.Optional[SortDirection]. Sort direction. Defaults to descending order if not provided.

            - page_token: typing.Optional[PageToken].
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/patient-payments/v4"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "patient_external_id": patient_external_id,
                    "claim_id": jsonable_encoder(claim_id),
                    "service_line_id": jsonable_encoder(service_line_id),
                    "billing_provider_id": jsonable_encoder(billing_provider_id),
                    "unattributed": unattributed,
                    "invoice_id": jsonable_encoder(invoice_id),
                    "sources": sources,
                    "sort": sort,
                    "sort_direction": sort_direction,
                    "page_token": page_token,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PatientPaymentsPage, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, patient_payment_id: PatientPaymentId) -> PatientPayment:
        """
        Retrieves a previously created patient payment by its `patient_payment_id`.

        Parameters:
            - patient_payment_id: PatientPaymentId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/patient-payments/v4/{patient_payment_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PatientPayment, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        amount_cents: int,
        payment_timestamp: typing.Optional[dt.datetime] = OMIT,
        payment_note: typing.Optional[str] = OMIT,
        patient_external_id: PatientExternalId,
        allocations: typing.List[AllocationCreate],
        invoice: typing.Optional[InvoiceId] = OMIT,
    ) -> PatientPayment:
        """
        Creates a new patient payment record and returns the newly created PatientPayment object.
        The allocations can describe whether the payment is being applied toward a specific service line,
        claim, or billing provider.

        Parameters:
            - amount_cents: int.

            - payment_timestamp: typing.Optional[dt.datetime].

            - payment_note: typing.Optional[str].

            - patient_external_id: PatientExternalId.

            - allocations: typing.List[AllocationCreate].

            - invoice: typing.Optional[InvoiceId].
        """
        _request: typing.Dict[str, typing.Any] = {
            "amount_cents": amount_cents,
            "patient_external_id": patient_external_id,
            "allocations": allocations,
        }
        if payment_timestamp is not OMIT:
            _request["payment_timestamp"] = payment_timestamp
        if payment_note is not OMIT:
            _request["payment_note"] = payment_note
        if invoice is not OMIT:
            _request["invoice"] = invoice
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/patient-payments/v4"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PatientPayment, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        patient_payment_id: PatientPaymentId,
        *,
        payment_timestamp: typing.Optional[dt.datetime] = OMIT,
        payment_note: typing.Optional[NoteUpdate] = OMIT,
        invoice: typing.Optional[InvoiceUpdate] = OMIT,
    ) -> PatientPayment:
        """
        Updates the patient payment record matching the provided patient_payment_id.

        Parameters:
            - patient_payment_id: PatientPaymentId.

            - payment_timestamp: typing.Optional[dt.datetime].

            - payment_note: typing.Optional[NoteUpdate].

            - invoice: typing.Optional[InvoiceUpdate].
        """
        _request: typing.Dict[str, typing.Any] = {}
        if payment_timestamp is not OMIT:
            _request["payment_timestamp"] = payment_timestamp
        if payment_note is not OMIT:
            _request["payment_note"] = payment_note
        if invoice is not OMIT:
            _request["invoice"] = invoice
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/patient-payments/v4/{patient_payment_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PatientPayment, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, patient_payment_id: PatientPaymentId) -> None:
        """
        Deletes the patient payment record matching the provided patient_payment_id.

        Parameters:
            - patient_payment_id: PatientPaymentId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/patient-payments/v4/{patient_payment_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV4Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        patient_external_id: typing.Optional[PatientExternalId] = None,
        claim_id: typing.Optional[ClaimId] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        unattributed: typing.Optional[bool] = None,
        invoice_id: typing.Optional[InvoiceId] = None,
        sources: typing.Optional[typing.Union[PatientTransactionSource, typing.List[PatientTransactionSource]]] = None,
        sort: typing.Optional[PatientPaymentSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
    ) -> PatientPaymentsPage:
        """
        Returns all patient payments satisfying the search criteria AND whose organization_id matches
        the current organization_id of the authenticated user.

        Parameters:
            - limit: typing.Optional[int]. Defaults to 100. The value must be greater than 0 and less than 1000.

            - patient_external_id: typing.Optional[PatientExternalId].

            - claim_id: typing.Optional[ClaimId].

            - service_line_id: typing.Optional[ServiceLineId].

            - billing_provider_id: typing.Optional[ProviderId].

            - unattributed: typing.Optional[bool]. returns payments with unattributed allocations if set to true

            - invoice_id: typing.Optional[InvoiceId].

            - sources: typing.Optional[typing.Union[PatientTransactionSource, typing.List[PatientTransactionSource]]].

            - sort: typing.Optional[PatientPaymentSortField]. Defaults to payment_timestamp

            - sort_direction: typing.Optional[SortDirection]. Sort direction. Defaults to descending order if not provided.

            - page_token: typing.Optional[PageToken].
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/patient-payments/v4"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "patient_external_id": patient_external_id,
                    "claim_id": jsonable_encoder(claim_id),
                    "service_line_id": jsonable_encoder(service_line_id),
                    "billing_provider_id": jsonable_encoder(billing_provider_id),
                    "unattributed": unattributed,
                    "invoice_id": jsonable_encoder(invoice_id),
                    "sources": sources,
                    "sort": sort,
                    "sort_direction": sort_direction,
                    "page_token": page_token,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PatientPaymentsPage, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, patient_payment_id: PatientPaymentId) -> PatientPayment:
        """
        Retrieves a previously created patient payment by its `patient_payment_id`.

        Parameters:
            - patient_payment_id: PatientPaymentId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/patient-payments/v4/{patient_payment_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PatientPayment, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        amount_cents: int,
        payment_timestamp: typing.Optional[dt.datetime] = OMIT,
        payment_note: typing.Optional[str] = OMIT,
        patient_external_id: PatientExternalId,
        allocations: typing.List[AllocationCreate],
        invoice: typing.Optional[InvoiceId] = OMIT,
    ) -> PatientPayment:
        """
        Creates a new patient payment record and returns the newly created PatientPayment object.
        The allocations can describe whether the payment is being applied toward a specific service line,
        claim, or billing provider.

        Parameters:
            - amount_cents: int.

            - payment_timestamp: typing.Optional[dt.datetime].

            - payment_note: typing.Optional[str].

            - patient_external_id: PatientExternalId.

            - allocations: typing.List[AllocationCreate].

            - invoice: typing.Optional[InvoiceId].
        """
        _request: typing.Dict[str, typing.Any] = {
            "amount_cents": amount_cents,
            "patient_external_id": patient_external_id,
            "allocations": allocations,
        }
        if payment_timestamp is not OMIT:
            _request["payment_timestamp"] = payment_timestamp
        if payment_note is not OMIT:
            _request["payment_note"] = payment_note
        if invoice is not OMIT:
            _request["invoice"] = invoice
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/patient-payments/v4"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PatientPayment, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        patient_payment_id: PatientPaymentId,
        *,
        payment_timestamp: typing.Optional[dt.datetime] = OMIT,
        payment_note: typing.Optional[NoteUpdate] = OMIT,
        invoice: typing.Optional[InvoiceUpdate] = OMIT,
    ) -> PatientPayment:
        """
        Updates the patient payment record matching the provided patient_payment_id.

        Parameters:
            - patient_payment_id: PatientPaymentId.

            - payment_timestamp: typing.Optional[dt.datetime].

            - payment_note: typing.Optional[NoteUpdate].

            - invoice: typing.Optional[InvoiceUpdate].
        """
        _request: typing.Dict[str, typing.Any] = {}
        if payment_timestamp is not OMIT:
            _request["payment_timestamp"] = payment_timestamp
        if payment_note is not OMIT:
            _request["payment_note"] = payment_note
        if invoice is not OMIT:
            _request["invoice"] = invoice
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/patient-payments/v4/{patient_payment_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PatientPayment, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, patient_payment_id: PatientPaymentId) -> None:
        """
        Deletes the patient payment record matching the provided patient_payment_id.

        Parameters:
            - patient_payment_id: PatientPaymentId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/patient-payments/v4/{patient_payment_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .in_network_rendering_providers_detail import InNetworkRenderingProvidersDetail
from .indeterminate_network_status import IndeterminateNetworkStatus
from .out_of_network_status import OutOfNetworkStatus


class ComputeAllInNetworkRenderingProvidersResult_RenderingProviders(InNetworkRenderingProvidersDetail):
    type: typing_extensions.Literal["rendering_providers"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ComputeAllInNetworkRenderingProvidersResult_Indeterminate(IndeterminateNetworkStatus):
    type: typing_extensions.Literal["indeterminate"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ComputeAllInNetworkRenderingProvidersResult_OutOfNetwork(OutOfNetworkStatus):
    type: typing_extensions.Literal["out_of_network"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


ComputeAllInNetworkRenderingProvidersResult = typing.Union[
    ComputeAllInNetworkRenderingProvidersResult_RenderingProviders,
    ComputeAllInNetworkRenderingProvidersResult_Indeterminate,
    ComputeAllInNetworkRenderingProvidersResult_OutOfNetwork,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_dict import remove_none_from_dict
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.claim_id import ClaimId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.provider_id import ProviderId
from ....commons.types.service_line_id import ServiceLineId
from ....commons.types.sort_direction import SortDirection
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from ....financials.types.note_update import NoteUpdate
from ....financials.types.refund_reason_update import RefundReasonUpdate
from ....payers.resources.v_3.types.payer_uuid import PayerUuid
from .types.insurance_refund import InsuranceRefund
from .types.insurance_refund_create import InsuranceRefundCreate
from .types.insurance_refund_id import InsuranceRefundId
from .types.insurance_refund_sort_field import InsuranceRefundSortField
from .types.insurance_refunds_page import InsuranceRefundsPage

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        payer_uuid: typing.Optional[PayerUuid] = None,
        claim_id: typing.Optional[ClaimId] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        sort: typing.Optional[InsuranceRefundSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
    ) -> InsuranceRefundsPage:
        """
        Returns all insurance refunds satisfying the search criteria AND whose organization_id matches
        the current organization_id of the authenticated user.

        Parameters:
            - limit: typing.Optional[int]. Defaults to 100. The value must be greater than 0 and less than 1000.

            - payer_uuid: typing.Optional[PayerUuid].

            - claim_id: typing.Optional[ClaimId].

            - service_line_id: typing.Optional[ServiceLineId].

            - billing_provider_id: typing.Optional[ProviderId].

            - sort: typing.Optional[InsuranceRefundSortField]. Defaults to refund_timestamp

            - sort_direction: typing.Optional[SortDirection]. Sort direction. Defaults to descending order if not provided.

            - page_token: typing.Optional[PageToken].
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/insurance-refunds/v1"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "payer_uuid": jsonable_encoder(payer_uuid),
                    "claim_id": jsonable_encoder(claim_id),
                    "service_line_id": jsonable_encoder(service_line_id),
                    "billing_provider_id": jsonable_encoder(billing_provider_id),
                    "sort": sort,
                    "sort_direction": sort_direction,
                    "page_token": page_token,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceRefundsPage, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, insurance_refund_id: InsuranceRefundId) -> InsuranceRefund:
        """
        Retrieves a previously created insurance refund by its `insurance_refund_id`.
        If the refund does not exist, a `403` will be thrown.

        Parameters:
            - insurance_refund_id: InsuranceRefundId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/insurance-refunds/v1/{insurance_refund_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceRefund, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, *, request: InsuranceRefundCreate) -> InsuranceRefund:
        """
        Creates a new insurance refund record and returns the newly created `InsuranceRefund` object.
        The allocations can describe whether the refund is being applied toward a specific service line,
        claim, or billing provider.

        Parameters:
            - request: InsuranceRefundCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/insurance-refunds/v1"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceRefund, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        insurance_refund_id: InsuranceRefundId,
        *,
        refund_timestamp: typing.Optional[dt.datetime] = OMIT,
        refund_note: typing.Optional[NoteUpdate] = OMIT,
        refund_reason: typing.Optional[RefundReasonUpdate] = OMIT,
    ) -> InsuranceRefund:
        """
        Updates the patient refund record matching the provided insurance_refund_id. If updating the refund amount,
        then the allocations must be appropriately updated as well.

        Parameters:
            - insurance_refund_id: InsuranceRefundId.

            - refund_timestamp: typing.Optional[dt.datetime].

            - refund_note: typing.Optional[NoteUpdate].

            - refund_reason: typing.Optional[RefundReasonUpdate].
        """
        _request: typing.Dict[str, typing.Any] = {}
        if refund_timestamp is not OMIT:
            _request["refund_timestamp"] = refund_timestamp
        if refund_note is not OMIT:
            _request["refund_note"] = refund_note
        if refund_reason is not OMIT:
            _request["refund_reason"] = refund_reason
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/insurance-refunds/v1/{insurance_refund_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceRefund, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, insurance_refund_id: InsuranceRefundId) -> None:
        """
        Deletes the insurance refund record matching the provided `insurance_refund_id`.
        If the matching record's organization_id does not match the authenticated user's
        current organization_id, then a response code of `403` will be returned.

        Parameters:
            - insurance_refund_id: InsuranceRefundId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/insurance-refunds/v1/{insurance_refund_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        payer_uuid: typing.Optional[PayerUuid] = None,
        claim_id: typing.Optional[ClaimId] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        sort: typing.Optional[InsuranceRefundSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
    ) -> InsuranceRefundsPage:
        """
        Returns all insurance refunds satisfying the search criteria AND whose organization_id matches
        the current organization_id of the authenticated user.

        Parameters:
            - limit: typing.Optional[int]. Defaults to 100. The value must be greater than 0 and less than 1000.

            - payer_uuid: typing.Optional[PayerUuid].

            - claim_id: typing.Optional[ClaimId].

            - service_line_id: typing.Optional[ServiceLineId].

            - billing_provider_id: typing.Optional[ProviderId].

            - sort: typing.Optional[InsuranceRefundSortField]. Defaults to refund_timestamp

            - sort_direction: typing.Optional[SortDirection]. Sort direction. Defaults to descending order if not provided.

            - page_token: typing.Optional[PageToken].
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/insurance-refunds/v1"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "payer_uuid": jsonable_encoder(payer_uuid),
                    "claim_id": jsonable_encoder(claim_id),
                    "service_line_id": jsonable_encoder(service_line_id),
                    "billing_provider_id": jsonable_encoder(billing_provider_id),
                    "sort": sort,
                    "sort_direction": sort_direction,
                    "page_token": page_token,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceRefundsPage, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, insurance_refund_id: InsuranceRefundId) -> InsuranceRefund:
        """
        Retrieves a previously created insurance refund by its `insurance_refund_id`.
        If the refund does not exist, a `403` will be thrown.

        Parameters:
            - insurance_refund_id: InsuranceRefundId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/insurance-refunds/v1/{insurance_refund_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceRefund, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(self, *, request: InsuranceRefundCreate) -> InsuranceRefund:
        """
        Creates a new insurance refund record and returns the newly created `InsuranceRefund` object.
        The allocations can describe whether the refund is being applied toward a specific service line,
        claim, or billing provider.

        Parameters:
            - request: InsuranceRefundCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/insurance-refunds/v1"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceRefund, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        insurance_refund_id: InsuranceRefundId,
        *,
        refund_timestamp: typing.Optional[dt.datetime] = OMIT,
        refund_note: typing.Optional[NoteUpdate] = OMIT,
        refund_reason: typing.Optional[RefundReasonUpdate] = OMIT,
    ) -> InsuranceRefund:
        """
        Updates the patient refund record matching the provided insurance_refund_id. If updating the refund amount,
        then the allocations must be appropriately updated as well.

        Parameters:
            - insurance_refund_id: InsuranceRefundId.

            - refund_timestamp: typing.Optional[dt.datetime].

            - refund_note: typing.Optional[NoteUpdate].

            - refund_reason: typing.Optional[RefundReasonUpdate].
        """
        _request: typing.Dict[str, typing.Any] = {}
        if refund_timestamp is not OMIT:
            _request["refund_timestamp"] = refund_timestamp
        if refund_note is not OMIT:
            _request["refund_note"] = refund_note
        if refund_reason is not OMIT:
            _request["refund_reason"] = refund_reason
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/insurance-refunds/v1/{insurance_refund_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InsuranceRefund, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, insurance_refund_id: InsuranceRefundId) -> None:
        """
        Deletes the insurance refund record matching the provided `insurance_refund_id`.
        If the matching record's organization_id does not match the authenticated user's
        current organization_id, then a response code of `403` will be returned.

        Parameters:
            - insurance_refund_id: InsuranceRefundId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/insurance-refunds/v1/{insurance_refund_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .refund_reason import RefundReason

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class RefundReasonUpdate_Set(pydantic.BaseModel):
    type: typing_extensions.Literal["set"]
    value: RefundReason

    class Config:
        frozen = True
        smart_union = True


class RefundReasonUpdate_Remove(pydantic.BaseModel):
    type: typing_extensions.Literal["remove"]

    class Config:
        frozen = True
        smart_union = True


RefundReasonUpdate = typing.Union[RefundReasonUpdate_Set, RefundReasonUpdate_Remove]



# This file was auto-generated by Fern from our API Definition.

from .entity_not_found_error import EntityNotFoundError
from .http_request_validation_error import HttpRequestValidationError
from .http_request_validations_error import HttpRequestValidationsError
from .http_service_unavailable_error import HttpServiceUnavailableError
from .organization_not_authorized_error import OrganizationNotAuthorizedError
from .unauthorized_error import UnauthorizedError
from .unprocessable_entity_error import UnprocessableEntityError
from .updates_disabled_due_to_external_system_integration_error import (
    UpdatesDisabledDueToExternalSystemIntegrationError,
)

__all__ = [
    "EntityNotFoundError",
    "HttpRequestValidationError",
    "HttpRequestValidationsError",
    "HttpServiceUnavailableError",
    "OrganizationNotAuthorizedError",
    "UnauthorizedError",
    "UnprocessableEntityError",
    "UpdatesDisabledDueToExternalSystemIntegrationError",
]



# This file was auto-generated by Fern from our API Definition.

from . import (
    auth,
    billing_notes,
    claim_submission,
    claims,
    commons,
    contracts,
    diagnoses,
    eligibility,
    encounter_providers,
    encounters,
    era,
    era_commons,
    expected_network_status,
    exports,
    financials,
    guarantor,
    identifiers,
    individual,
    insurance_adjudications,
    insurance_cards,
    insurance_payments,
    insurance_refunds,
    invoices,
    organization_providers,
    organization_service_facilities,
    patient_payments,
    patient_refunds,
    payers,
    remits,
    service_facility,
    service_lines,
    tags,
    tasks,
    write_offs,
    x_12,
)
from .claims import Claim, ClaimStatus
from .commons import (
    ClaimAdjustmentGroupCodes,
    ClaimId,
    ClaimSubmissionPayerResponsibilityType,
    Date,
    DateRangeOptionalEnd,
    Decimal,
    DelayReasonCode,
    Email,
    EmrPayerCrosswalk,
    EncounterExternalId,
    EncounterId,
    EntityNotFoundError,
    EntityNotFoundErrorMessage,
    ErrorMessage,
    FacilityTypeCode,
    HttpRequestValidationError,
    HttpRequestValidationsError,
    HttpServiceUnavailableError,
    HttpServiceUnavailableErrorMessage,
    InsuranceTypeCode,
    InvoiceId,
    LinkUrl,
    Npi,
    OrganizationId,
    OrganizationNotAuthorizedError,
    OrganizationNotAuthorizedErrorMessage,
    PageToken,
    PatientExternalId,
    PatientRelationshipToInsuredCodeAll,
    PhoneNumber,
    PhoneNumberType,
    ProcedureModifier,
    ProviderId,
    RegionNational,
    RegionStates,
    Regions,
    Regions_National,
    Regions_States,
    RemovableDateRangeOptionalEnd,
    RemovableDateRangeOptionalEnd_DateRange,
    RemovableDateRangeOptionalEnd_Remove,
    RequestValidationError,
    ResourcePage,
    ServiceLineId,
    ServiceLineUnits,
    SortDirection,
    SourceOfPaymentCode,
    State,
    StreetAddressBase,
    StreetAddressLongZip,
    StreetAddressShortZip,
    TaskAssignmentId,
    TaskId,
    TaskNoteId,
    TaxId,
    UnauthorizedError,
    UnauthorizedErrorMessage,
    UnprocessableEntityError,
    UnprocessableEntityErrorMessage,
    UpdatesDisabledDueToExternalSystemIntegrationError,
    UpdatesDisabledDueToExternalSystemIntegrationErrorMessage,
    UserId,
    WorkQueueId,
)
from .diagnoses import Diagnosis, DiagnosisCreate, DiagnosisId, DiagnosisTypeCode, StandaloneDiagnosisCreate
from .era import Era, EraBase, EraId, EraNotFullyProcessedError, EraNotFullyProcessedErrorMessage
from .era_commons import ClaimStatusCodeCreate
from .financials import (
    AccountType,
    Allocation,
    AllocationCreate,
    AllocationTarget,
    AllocationTargetCreate,
    AllocationTargetCreate_BillingProviderById,
    AllocationTargetCreate_ClaimById,
    AllocationTargetCreate_ServiceLineById,
    AllocationTargetCreate_Unattributed,
    AllocationTarget_BillingProviderId,
    AllocationTarget_Claim,
    AllocationTarget_ServiceLine,
    AllocationTarget_Unattributed,
    BillingProviderAllocationTarget,
    ClaimAllocationTarget,
    InvoiceUpdate,
    InvoiceUpdate_Remove,
    InvoiceUpdate_Set,
    NoteUpdate,
    NoteUpdate_Remove,
    NoteUpdate_Set,
    PatientTransactionSource,
    RefundReason,
    RefundReasonUpdate,
    RefundReasonUpdate_Remove,
    RefundReasonUpdate_Set,
    ServiceLineAllocationTarget,
)
from .identifiers import (
    Identifier,
    IdentifierBase,
    IdentifierCode,
    IdentifierCreate,
    IdentifierId,
    IdentifierUpdate,
    IdentifierValue,
    IdentifierValue_MedicaidProviderIdentifier,
    IdentifierValue_MedicareProviderIdentifier,
    MedicaidProviderIdentifier,
    MedicareProviderIdentifier,
    UpdatableIdentifier,
    UpdatableIdentifier_Add,
    UpdatableIdentifier_Remove,
    UpdatableIdentifier_Update,
)
from .individual import (
    Gender,
    IndividualBase,
    IndividualId,
    Patient,
    PatientBase,
    PatientCreate,
    Subscriber,
    SubscriberBase,
    SubscriberCreate,
)
from .invoices import Invoice, InvoiceItem, InvoiceStatus
from .service_facility import EncounterServiceFacility, EncounterServiceFacilityBase, ServiceFacilityId
from .tags import Tag, TagColorEnum, TagCreate, TagId

__all__ = [
    "AccountType",
    "Allocation",
    "AllocationCreate",
    "AllocationTarget",
    "AllocationTargetCreate",
    "AllocationTargetCreate_BillingProviderById",
    "AllocationTargetCreate_ClaimById",
    "AllocationTargetCreate_ServiceLineById",
    "AllocationTargetCreate_Unattributed",
    "AllocationTarget_BillingProviderId",
    "AllocationTarget_Claim",
    "AllocationTarget_ServiceLine",
    "AllocationTarget_Unattributed",
    "BillingProviderAllocationTarget",
    "Claim",
    "ClaimAdjustmentGroupCodes",
    "ClaimAllocationTarget",
    "ClaimId",
    "ClaimStatus",
    "ClaimStatusCodeCreate",
    "ClaimSubmissionPayerResponsibilityType",
    "Date",
    "DateRangeOptionalEnd",
    "Decimal",
    "DelayReasonCode",
    "Diagnosis",
    "DiagnosisCreate",
    "DiagnosisId",
    "DiagnosisTypeCode",
    "Email",
    "EmrPayerCrosswalk",
    "EncounterExternalId",
    "EncounterId",
    "EncounterServiceFacility",
    "EncounterServiceFacilityBase",
    "EntityNotFoundError",
    "EntityNotFoundErrorMessage",
    "Era",
    "EraBase",
    "EraId",
    "EraNotFullyProcessedError",
    "EraNotFullyProcessedErrorMessage",
    "ErrorMessage",
    "FacilityTypeCode",
    "Gender",
    "HttpRequestValidationError",
    "HttpRequestValidationsError",
    "HttpServiceUnavailableError",
    "HttpServiceUnavailableErrorMessage",
    "Identifier",
    "IdentifierBase",
    "IdentifierCode",
    "IdentifierCreate",
    "IdentifierId",
    "IdentifierUpdate",
    "IdentifierValue",
    "IdentifierValue_MedicaidProviderIdentifier",
    "IdentifierValue_MedicareProviderIdentifier",
    "IndividualBase",
    "IndividualId",
    "InsuranceTypeCode",
    "Invoice",
    "InvoiceId",
    "InvoiceItem",
    "InvoiceStatus",
    "InvoiceUpdate",
    "InvoiceUpdate_Remove",
    "InvoiceUpdate_Set",
    "LinkUrl",
    "MedicaidProviderIdentifier",
    "MedicareProviderIdentifier",
    "NoteUpdate",
    "NoteUpdate_Remove",
    "NoteUpdate_Set",
    "Npi",
    "OrganizationId",
    "OrganizationNotAuthorizedError",
    "OrganizationNotAuthorizedErrorMessage",
    "PageToken",
    "Patient",
    "PatientBase",
    "PatientCreate",
    "PatientExternalId",
    "PatientRelationshipToInsuredCodeAll",
    "PatientTransactionSource",
    "PhoneNumber",
    "PhoneNumberType",
    "ProcedureModifier",
    "ProviderId",
    "RefundReason",
    "RefundReasonUpdate",
    "RefundReasonUpdate_Remove",
    "RefundReasonUpdate_Set",
    "RegionNational",
    "RegionStates",
    "Regions",
    "Regions_National",
    "Regions_States",
    "RemovableDateRangeOptionalEnd",
    "RemovableDateRangeOptionalEnd_DateRange",
    "RemovableDateRangeOptionalEnd_Remove",
    "RequestValidationError",
    "ResourcePage",
    "ServiceFacilityId",
    "ServiceLineAllocationTarget",
    "ServiceLineId",
    "ServiceLineUnits",
    "SortDirection",
    "SourceOfPaymentCode",
    "StandaloneDiagnosisCreate",
    "State",
    "StreetAddressBase",
    "StreetAddressLongZip",
    "StreetAddressShortZip",
    "Subscriber",
    "SubscriberBase",
    "SubscriberCreate",
    "Tag",
    "TagColorEnum",
    "TagCreate",
    "TagId",
    "TaskAssignmentId",
    "TaskId",
    "TaskNoteId",
    "TaxId",
    "UnauthorizedError",
    "UnauthorizedErrorMessage",
    "UnprocessableEntityError",
    "UnprocessableEntityErrorMessage",
    "UpdatableIdentifier",
    "UpdatableIdentifier_Add",
    "UpdatableIdentifier_Remove",
    "UpdatableIdentifier_Update",
    "UpdatesDisabledDueToExternalSystemIntegrationError",
    "UpdatesDisabledDueToExternalSystemIntegrationErrorMessage",
    "UserId",
    "WorkQueueId",
    "auth",
    "billing_notes",
    "claim_submission",
    "claims",
    "commons",
    "contracts",
    "diagnoses",
    "eligibility",
    "encounter_providers",
    "encounters",
    "era",
    "era_commons",
    "expected_network_status",
    "exports",
    "financials",
    "guarantor",
    "identifiers",
    "individual",
    "insurance_adjudications",
    "insurance_cards",
    "insurance_payments",
    "insurance_refunds",
    "invoices",
    "organization_providers",
    "organization_service_facilities",
    "patient_payments",
    "patient_refunds",
    "payers",
    "remits",
    "service_facility",
    "service_lines",
    "tags",
    "tasks",
    "write_offs",
    "x_12",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.remove_none_from_dict import remove_none_from_dict
from ....commons.errors.http_request_validations_error import HttpRequestValidationsError
from ....commons.types.error_message import ErrorMessage
from ....commons.types.request_validation_error import RequestValidationError
from .errors.export_date_too_early_error import ExportDateTooEarlyError
from .errors.export_files_unavailable_error import ExportFilesUnavailableError
from .errors.export_not_yet_available_error import ExportNotYetAvailableError
from .errors.missing_daily_incremental_export_file_error import MissingDailyIncrementalExportFileError
from .types.get_exports_response import GetExportsResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class V3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_exports(self, *, start_date: dt.date, end_date: dt.date) -> GetExportsResponse:
        """
        Retrieve CSV-formatted reports on claim submissions and outcomes. This endpoint returns Export objects that contain an
        authenticated URL to a customer's reports with a 2min TTL. The schema for the CSV export can be found [here](https://app.joincandidhealth.com/files/exports_schema.csv).

        **Schema changes:** Changing column order, removing columns, or changing the name of a column is considered a
        [Breaking Change](../../../api-principles/breaking-changes). Adding new columns to the end of the Exports file is not considered a
        Breaking Change and happens periodically. For this reason, it is important that any downstream automation or processes built on top
        of Candid Health's export files be resilient to the addition of new columns at the end of the file.

        **SLA guarantees:** Files for a given date are guaranteed to be available after 3 business days. For example, Friday's file will be
        available by Wednesday at the latest. If file generation is still in progress upon request before 3 business days have passed, the
        caller will receive a 422 response. If the file has already been generated, it will be served. Please email
        our [Support team](mailto:support@joincandidhealth.com) with any data requests outside of these stated guarantees.

        Parameters:
            - start_date: dt.date. Beginning date of claim versions returned in the export, ISO 8601 date e.g. 2019-08-24.
                                   Must be at least 1 calendar day in the past. Cannot be earlier than 2022-10-07.
            - end_date: dt.date. Ending date of claim versions returned in the export, ISO 8601 date; e.g. 2019-08-24.
                                 Must be within 30 days of start_date.---
        import datetime

        from candid.client import CandidApi

        client = CandidApi(
            token="YOUR_TOKEN",
        )
        client.exports.v_3.get_exports(
            start_date=datetime.date.fromisoformat(
                "2023-10-01",
            ),
            end_date=datetime.date.fromisoformat(
                "2023-10-02",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/exports/v3"),
            params=remove_none_from_dict({"start_date": str(start_date), "end_date": str(end_date)}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetExportsResponse, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    pydantic.parse_obj_as(typing.List[RequestValidationError], _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "ExportFilesUnavailableError":
                raise ExportFilesUnavailableError(
                    pydantic.parse_obj_as(ErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "MissingDailyIncrementalExportFileError":
                raise MissingDailyIncrementalExportFileError(
                    pydantic.parse_obj_as(ErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "ExportNotYetAvailableError":
                raise ExportNotYetAvailableError(
                    pydantic.parse_obj_as(ErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "ExportDateTooEarlyError":
                raise ExportDateTooEarlyError(
                    pydantic.parse_obj_as(ErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_exports(self, *, start_date: dt.date, end_date: dt.date) -> GetExportsResponse:
        """
        Retrieve CSV-formatted reports on claim submissions and outcomes. This endpoint returns Export objects that contain an
        authenticated URL to a customer's reports with a 2min TTL. The schema for the CSV export can be found [here](https://app.joincandidhealth.com/files/exports_schema.csv).

        **Schema changes:** Changing column order, removing columns, or changing the name of a column is considered a
        [Breaking Change](../../../api-principles/breaking-changes). Adding new columns to the end of the Exports file is not considered a
        Breaking Change and happens periodically. For this reason, it is important that any downstream automation or processes built on top
        of Candid Health's export files be resilient to the addition of new columns at the end of the file.

        **SLA guarantees:** Files for a given date are guaranteed to be available after 3 business days. For example, Friday's file will be
        available by Wednesday at the latest. If file generation is still in progress upon request before 3 business days have passed, the
        caller will receive a 422 response. If the file has already been generated, it will be served. Please email
        our [Support team](mailto:support@joincandidhealth.com) with any data requests outside of these stated guarantees.

        Parameters:
            - start_date: dt.date. Beginning date of claim versions returned in the export, ISO 8601 date e.g. 2019-08-24.
                                   Must be at least 1 calendar day in the past. Cannot be earlier than 2022-10-07.
            - end_date: dt.date. Ending date of claim versions returned in the export, ISO 8601 date; e.g. 2019-08-24.
                                 Must be within 30 days of start_date.---
        import datetime

        from candid.client import AsyncCandidApi

        client = AsyncCandidApi(
            token="YOUR_TOKEN",
        )
        await client.exports.v_3.get_exports(
            start_date=datetime.date.fromisoformat(
                "2023-10-01",
            ),
            end_date=datetime.date.fromisoformat(
                "2023-10-02",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/exports/v3"),
            params=remove_none_from_dict({"start_date": str(start_date), "end_date": str(end_date)}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetExportsResponse, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    pydantic.parse_obj_as(typing.List[RequestValidationError], _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "ExportFilesUnavailableError":
                raise ExportFilesUnavailableError(
                    pydantic.parse_obj_as(ErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "MissingDailyIncrementalExportFileError":
                raise MissingDailyIncrementalExportFileError(
                    pydantic.parse_obj_as(ErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "ExportNotYetAvailableError":
                raise ExportNotYetAvailableError(
                    pydantic.parse_obj_as(ErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "ExportDateTooEarlyError":
                raise ExportDateTooEarlyError(
                    pydantic.parse_obj_as(ErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .insurance_write_off import InsuranceWriteOff
from .patient_write_off import PatientWriteOff


class WriteOff_Patient(PatientWriteOff):
    type: typing_extensions.Literal["patient"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class WriteOff_Insurance(InsuranceWriteOff):
    type: typing_extensions.Literal["insurance"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


WriteOff = typing.Union[WriteOff_Patient, WriteOff_Insurance]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .....commons.types.insurance_type_code import InsuranceTypeCode

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class InsuranceTypes_AllApply(pydantic.BaseModel):
    type: typing_extensions.Literal["allApply"]

    class Config:
        frozen = True
        smart_union = True


class InsuranceTypes_NoneApply(pydantic.BaseModel):
    type: typing_extensions.Literal["noneApply"]

    class Config:
        frozen = True
        smart_union = True


class InsuranceTypes_TheseApply(pydantic.BaseModel):
    type: typing_extensions.Literal["theseApply"]
    value: typing.List[InsuranceTypeCode]

    class Config:
        frozen = True
        smart_union = True


InsuranceTypes = typing.Union[InsuranceTypes_AllApply, InsuranceTypes_NoneApply, InsuranceTypes_TheseApply]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .claim_submission_record_create import ClaimSubmissionRecordCreate

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ExternalClaimSubmissionCreate(pydantic.BaseModel):
    """
    import datetime

    from candid import ClaimSubmissionPayerResponsibilityType
    from candid.resources.claim_submission.v_1 import (
        ClaimFrequencyTypeCode, ClaimSubmissionRecordCreate,
        ExternalClaimSubmissionCreate)

    ExternalClaimSubmissionCreate(claim_created_at=datetime.datetime.fromisoformat("2023-01-01 12:00:00+00:00", ), patient_control_number="PATIENT_CONTROL_NUMBER", submission_records=[ClaimSubmissionRecordCreate(submitted_at=datetime.datetime.fromisoformat("2023-01-01 13:00:00+00:00", ), claim_frequency_code=ClaimFrequencyTypeCode.1, payer_responsibility=ClaimSubmissionPayerResponsibilityType.PRIMARY, ), ClaimSubmissionRecordCreate(submitted_at=datetime.datetime.fromisoformat("2023-01-04 12:00:00+00:00", ), claim_frequency_code=ClaimFrequencyTypeCode.6, payer_responsibility=ClaimSubmissionPayerResponsibilityType.PRIMARY, )], )
    """

    claim_created_at: dt.datetime = pydantic.Field(description="When the claim was created in the external system.")
    patient_control_number: str = pydantic.Field(
        description=(
            "The Patient Control Number sent on the claim to the payer. To guarantee compatibility with all payers, this field must consist\n"
            "only of uppercase letters and numbers and be no more than 14 characters long.\n"
        )
    )
    submission_records: typing.List[ClaimSubmissionRecordCreate] = pydantic.Field(
        description=(
            "A successful claim submission record will be created for each value provided.\n"
            "An empty list may be provided for cases where the claim originated in an external system but was never submitted to a payer.\n"
        )
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class SynchronicityType(str, enum.Enum):
    SYNCHRONOUS = "Synchronous"
    ASYNCHRONOUS = "Asynchronous"

    def visit(
        self, synchronous: typing.Callable[[], T_Result], asynchronous: typing.Callable[[], T_Result]
    ) -> T_Result:
        if self is SynchronicityType.SYNCHRONOUS:
            return synchronous()
        if self is SynchronicityType.ASYNCHRONOUS:
            return asynchronous()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.encounter_external_id import EncounterExternalId
from .....commons.types.organization_id import OrganizationId
from .....commons.types.patient_external_id import PatientExternalId
from .....commons.types.service_line_id import ServiceLineId
from .patient_payment_id import PatientPaymentId
from .patient_payment_source import PatientPaymentSource
from .patient_payment_status import PatientPaymentStatus

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class PatientPayment(pydantic.BaseModel):
    """
    import datetime
    import uuid

    from candid.resources.patient_payments.v_3 import (
        PatientPayment,
        PatientPaymentSource,
        PatientPaymentStatus,
    )

    PatientPayment(
        patient_payment_id="CF237BE1-E793-4BBF-8958-61D5179D1D0D",
        organization_id=uuid.UUID(
            "0788ca2a-b20d-4b8e-b8d4-07fa0b3b4907",
        ),
        source_internal_id="D1A76039-D5C5-4323-A2FC-B7C8B6AEF6A4",
        source=PatientPaymentSource.MANUAL_ENTRY,
        amount_cents=2000,
        payment_timestamp=datetime.datetime.fromisoformat(
            "2023-01-01 00:00:00+00:00",
        ),
        status=PatientPaymentStatus.PENDING,
        payment_name="John Doe",
        payment_note="test payment note",
        patient_external_id="B7437260-D6B4-48CF-B9D7-753C09F34E76",
        encounter_external_id="0F26B9C3-199F-4CBB-A166-B87EA7C631BB",
        service_line_id=uuid.UUID(
            "b557dc86-c629-478c-850a-02d45ac11783",
        ),
    )
    """

    patient_payment_id: PatientPaymentId
    organization_id: OrganizationId
    source_internal_id: str
    source: PatientPaymentSource
    amount_cents: int
    payment_timestamp: typing.Optional[dt.datetime] = None
    status: typing.Optional[PatientPaymentStatus] = None
    payment_name: typing.Optional[str] = None
    payment_note: typing.Optional[str] = None
    patient_external_id: typing.Optional[PatientExternalId] = None
    encounter_external_id: typing.Optional[EncounterExternalId] = None
    service_line_id: typing.Optional[ServiceLineId] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.resource_page import ResourcePage
from .payer import Payer


class PayerPage(ResourcePage):
    """
    import uuid

    from candid.resources.payers.v_3 import Payer, PayerPage

    PayerPage(
        items=[
            Payer(
                payer_uuid=uuid.UUID(
                    "a6431fd2-0712-4714-b1b1-dd094daf9f42",
                ),
                payer_id="12345",
                payer_name="Payer Name",
            )
        ],
    )
    """

    items: typing.List[Payer]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ..commons.types.street_address_long_zip import StreetAddressLongZip
from .types.encounter_service_facility import EncounterServiceFacility
from .types.service_facility_id import ServiceFacilityId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ServiceFacilityClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def update(
        self,
        service_facility_id: ServiceFacilityId,
        *,
        organization_name: typing.Optional[str] = OMIT,
        npi: typing.Optional[str] = OMIT,
        address: typing.Optional[StreetAddressLongZip] = OMIT,
    ) -> EncounterServiceFacility:
        """
        Parameters:
            - service_facility_id: ServiceFacilityId.

            - organization_name: typing.Optional[str].

            - npi: typing.Optional[str]. An NPI specific to the service facility if applicable, i.e. if it has one and is not under the billing provider's NPI.
                                         Box 32 section (a) of the CMS-1500 claim form.

            - address: typing.Optional[StreetAddressLongZip]. zip_plus_four_code is required for service facility address. When the zip_plus_four_code is not available use "9998" as per CMS documentation.

        """
        _request: typing.Dict[str, typing.Any] = {}
        if organization_name is not OMIT:
            _request["organization_name"] = organization_name
        if npi is not OMIT:
            _request["npi"] = npi
        if address is not OMIT:
            _request["address"] = address
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/service_facility/v2/{service_facility_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EncounterServiceFacility, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncServiceFacilityClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def update(
        self,
        service_facility_id: ServiceFacilityId,
        *,
        organization_name: typing.Optional[str] = OMIT,
        npi: typing.Optional[str] = OMIT,
        address: typing.Optional[StreetAddressLongZip] = OMIT,
    ) -> EncounterServiceFacility:
        """
        Parameters:
            - service_facility_id: ServiceFacilityId.

            - organization_name: typing.Optional[str].

            - npi: typing.Optional[str]. An NPI specific to the service facility if applicable, i.e. if it has one and is not under the billing provider's NPI.
                                         Box 32 section (a) of the CMS-1500 claim form.

            - address: typing.Optional[StreetAddressLongZip]. zip_plus_four_code is required for service facility address. When the zip_plus_four_code is not available use "9998" as per CMS documentation.

        """
        _request: typing.Dict[str, typing.Any] = {}
        if organization_name is not OMIT:
            _request["organization_name"] = organization_name
        if npi is not OMIT:
            _request["npi"] = npi
        if address is not OMIT:
            _request["address"] = address
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/service_facility/v2/{service_facility_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EncounterServiceFacility, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...commons.types.street_address_long_zip import StreetAddressLongZip

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class EncounterServiceFacilityBase(pydantic.BaseModel):
    """
    Encounter Service facility is typically the location a medical service was rendered, such as a provider office or hospital.
    For telehealth, service facility can represent the provider's location when the service was delivered (e.g., home),
    or the location where an in-person visit would have taken place, whichever is easier to identify.
    If the provider is in-network, service facility may be defined in payer contracts.
    Box 32 on the CMS-1500 claim form.
    Note that for an in-network claim to be successfully adjudicated, the service facility address listed on claims
    must match what was provided to the payer during the credentialing process.
    """

    organization_name: str
    npi: typing.Optional[str] = pydantic.Field(
        default=None,
        description=(
            "An NPI specific to the service facility if applicable, i.e. if it has one and is not under the billing provider's NPI.\n"
            "Box 32 section (a) of the CMS-1500 claim form.\n"
        ),
    )
    address: StreetAddressLongZip = pydantic.Field(
        description='zip_plus_four_code is required for service facility address. When the zip_plus_four_code is not available use "9998" as per CMS documentation.'
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .identifier_create import IdentifierCreate
from .identifier_id import IdentifierId
from .identifier_update import IdentifierUpdate

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class UpdatableIdentifier_Add(IdentifierCreate):
    type: typing_extensions.Literal["add"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class UpdatableIdentifier_Update(IdentifierUpdate):
    type: typing_extensions.Literal["update"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class UpdatableIdentifier_Remove(pydantic.BaseModel):
    type: typing_extensions.Literal["remove"]
    value: IdentifierId

    class Config:
        frozen = True
        smart_union = True


UpdatableIdentifier = typing.Union[UpdatableIdentifier_Add, UpdatableIdentifier_Update, UpdatableIdentifier_Remove]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .insurance_card_base import InsuranceCardBase
from .insurance_card_id import InsuranceCardId


class InsuranceCard(InsuranceCardBase):
    """
    import uuid

    from candid import InsuranceTypeCode, SourceOfPaymentCode
    from candid.resources.insurance_cards.v_2 import InsuranceCard

    InsuranceCard(insurance_card_id=uuid.UUID("ca5b7711-4419-4161-9b7c-3494ac40c8d4", ), member_id="E85313B4-0FFC-4119-8042-8161A4ECFF0A", payer_name="John Doe", payer_id="836DDAA6-863F-4020-ACCA-205A689F0002", rx_bin="610014", rx_pcn="MEDDPRIME", image_url_front="https://s3.amazonaws.com/front.jpg", image_url_back="https://s3.amazonaws.com/back.jpg", group_number="ABC12345", plan_name="Silver PPO Plan", plan_type=SourceOfPaymentCode.09, insurance_type=InsuranceTypeCode.12, )
    """

    insurance_card_id: InsuranceCardId
    member_id: str
    payer_name: str
    payer_id: str
    rx_bin: typing.Optional[str] = None
    rx_pcn: typing.Optional[str] = None
    image_url_front: typing.Optional[str] = None
    image_url_back: typing.Optional[str] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_dict import remove_none_from_dict
from ....commons.types.claim_id import ClaimId
from ....commons.types.page_token import PageToken
from ....commons.types.patient_external_id import PatientExternalId
from ....commons.types.provider_id import ProviderId
from ....commons.types.service_line_id import ServiceLineId
from ....commons.types.sort_direction import SortDirection
from ....financials.types.account_type import AccountType
from ....payers.resources.v_3.types.payer_uuid import PayerUuid
from .types.create_write_offs_response import CreateWriteOffsResponse
from .types.write_off import WriteOff
from .types.write_off_create import WriteOffCreate
from .types.write_off_id import WriteOffId
from .types.write_off_sort_field import WriteOffSortField
from .types.write_offs_page import WriteOffsPage

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        patient_external_id: typing.Optional[PatientExternalId] = None,
        payer_uuid: typing.Optional[PayerUuid] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        claim_id: typing.Optional[ClaimId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        sort: typing.Optional[WriteOffSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        account_types: typing.Optional[typing.Union[AccountType, typing.List[AccountType]]] = None,
    ) -> WriteOffsPage:
        """
        Returns all write-offs satisfying the search criteria.

        Parameters:
            - limit: typing.Optional[int]. Defaults to 100. The value must be greater than 0 and less than 1000.

            - patient_external_id: typing.Optional[PatientExternalId].

            - payer_uuid: typing.Optional[PayerUuid].

            - service_line_id: typing.Optional[ServiceLineId].

            - claim_id: typing.Optional[ClaimId].

            - billing_provider_id: typing.Optional[ProviderId].

            - sort: typing.Optional[WriteOffSortField]. Defaults to write_off_timestamp

            - sort_direction: typing.Optional[SortDirection]. Sort direction. Defaults to descending order if not provided.

            - page_token: typing.Optional[PageToken].

            - account_types: typing.Optional[typing.Union[AccountType, typing.List[AccountType]]]. Filters the returned values to include only the provided account types.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/write-offs/v1"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "patient_external_id": patient_external_id,
                    "payer_uuid": jsonable_encoder(payer_uuid),
                    "service_line_id": jsonable_encoder(service_line_id),
                    "claim_id": jsonable_encoder(claim_id),
                    "billing_provider_id": jsonable_encoder(billing_provider_id),
                    "sort": sort,
                    "sort_direction": sort_direction,
                    "page_token": page_token,
                    "account_types": account_types,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(WriteOffsPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, write_off_id: WriteOffId) -> WriteOff:
        """
        Retrieves a previously created write off by its `write_off_id`.

        Parameters:
            - write_off_id: WriteOffId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/write-offs/v1/{write_off_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(WriteOff, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, *, write_offs: typing.List[WriteOffCreate]) -> CreateWriteOffsResponse:
        """
        Creates one or many write-offs given a specific set of allocations.
        The allocations can describe whether the refund is being applied toward a specific service line,
        claim, or billing provider.

        Parameters:
            - write_offs: typing.List[WriteOffCreate].
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/write-offs/v1"),
            json=jsonable_encoder({"write_offs": write_offs}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateWriteOffsResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def revert(self, write_off_id: WriteOffId) -> WriteOff:
        """
        Reverts a write off given a `write_off_id`.

        Parameters:
            - write_off_id: WriteOffId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/write-offs/v1/{write_off_id}/revert"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(WriteOff, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        patient_external_id: typing.Optional[PatientExternalId] = None,
        payer_uuid: typing.Optional[PayerUuid] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        claim_id: typing.Optional[ClaimId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        sort: typing.Optional[WriteOffSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        account_types: typing.Optional[typing.Union[AccountType, typing.List[AccountType]]] = None,
    ) -> WriteOffsPage:
        """
        Returns all write-offs satisfying the search criteria.

        Parameters:
            - limit: typing.Optional[int]. Defaults to 100. The value must be greater than 0 and less than 1000.

            - patient_external_id: typing.Optional[PatientExternalId].

            - payer_uuid: typing.Optional[PayerUuid].

            - service_line_id: typing.Optional[ServiceLineId].

            - claim_id: typing.Optional[ClaimId].

            - billing_provider_id: typing.Optional[ProviderId].

            - sort: typing.Optional[WriteOffSortField]. Defaults to write_off_timestamp

            - sort_direction: typing.Optional[SortDirection]. Sort direction. Defaults to descending order if not provided.

            - page_token: typing.Optional[PageToken].

            - account_types: typing.Optional[typing.Union[AccountType, typing.List[AccountType]]]. Filters the returned values to include only the provided account types.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/write-offs/v1"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "patient_external_id": patient_external_id,
                    "payer_uuid": jsonable_encoder(payer_uuid),
                    "service_line_id": jsonable_encoder(service_line_id),
                    "claim_id": jsonable_encoder(claim_id),
                    "billing_provider_id": jsonable_encoder(billing_provider_id),
                    "sort": sort,
                    "sort_direction": sort_direction,
                    "page_token": page_token,
                    "account_types": account_types,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(WriteOffsPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, write_off_id: WriteOffId) -> WriteOff:
        """
        Retrieves a previously created write off by its `write_off_id`.

        Parameters:
            - write_off_id: WriteOffId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/write-offs/v1/{write_off_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(WriteOff, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(self, *, write_offs: typing.List[WriteOffCreate]) -> CreateWriteOffsResponse:
        """
        Creates one or many write-offs given a specific set of allocations.
        The allocations can describe whether the refund is being applied toward a specific service line,
        claim, or billing provider.

        Parameters:
            - write_offs: typing.List[WriteOffCreate].
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/write-offs/v1"),
            json=jsonable_encoder({"write_offs": write_offs}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateWriteOffsResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def revert(self, write_off_id: WriteOffId) -> WriteOff:
        """
        Reverts a write off given a `write_off_id`.

        Parameters:
            - write_off_id: WriteOffId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/write-offs/v1/{write_off_id}/revert"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(WriteOff, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)
